import { basename, join, resolve } from 'path';
import { Observable } from 'rxjs';
import { filter, map } from 'rxjs/operators';
import { log, Template, cli } from './common';
import * as middleware from './middleware';
export async function init() {
    log.info();
    log.info.gray(`👋`);
    const { tmpl, variables, dir } = await prepareTemplate({});
    if (!tmpl || !dir || !variables) {
        return;
    }
    const alerts$ = tmpl.events$.pipe(filter(e => e.type === 'TMPL/alert'), map(e => e.payload));
    const task = {
        title: 'Setup',
        task: () => new Observable(observer => {
            alerts$.subscribe(e => observer.next(e.message));
            (async () => {
                await tmpl.execute({ variables });
                observer.complete();
            })();
        }),
    };
    try {
        log.info();
        await cli.exec.tasks.run(task);
        log.info();
        logComplete({ dir });
    }
    catch (error) {
        log.error(error);
    }
    log.info();
}
async function prepareTemplate(args) {
    let { template, moduleName } = args;
    if (!moduleName) {
        const res = await cli.prompt.text({ message: 'Module name' });
        if (!res) {
            return {};
        }
        moduleName = res;
    }
    if (!template) {
        const res = await cli.prompt.radio({
            message: 'Template',
            items: [
                { name: '@platform toolchain', value: 'platform' },
            ],
        });
        if (!res) {
            return {};
        }
        template = res;
    }
    const rename = [{ from: 'pkg.json', to: 'package.json' }];
    const tmpl = Template
        .create(join(__dirname, '../templates/base'))
        .use(middleware.processPackage({ filename: 'pkg.json' }))
        .use(middleware.saveFile({ rename }))
        .use(middleware.npmInstall())
        .use(middleware.runInitCommand({ template, done: 'COMPLETE' }));
    const dir = resolve(`./${moduleName}`);
    const variables = {
        template,
        platform: ['ELECTRON', 'WEB'],
        moduleName,
        dir,
    };
    return { tmpl, variables, dir };
}
function logComplete(args) {
    const dir = basename(args.dir);
    log.info.gray('🖐  To start your development server:\n');
    log.info.cyan(`     cd ${log.white(dir)}`);
    log.info.cyan(`     yarn start`);
    log.info();
    log.info.gray('👉  To see all available UIHarness commands:\n');
    log.info.cyan(`     cd ${log.white(dir)}`);
    log.info.cyan(`     yarn ui`);
    log.info();
    log.info.gray(`See ${log.blue('https://uiharness.com')} for more.\n`);
    log.info();
}
